'$INCLUDE: 'FILEINC.BI'

'   Name: File Utility Library
' Author: Erik Eriksen
'   Date: 2020-05-25 (Latest Update: 2020-06-18)
'Purpose: This library contains sub routines used by the NE File Utility
'         application. Due to size constraints, they are loaded into a
'         separate QB Library.
'
' Running FileUtil.BAS requires the use of this library's compiled code
' and using the "/L" command parameter to include the library at start.
' Running the FileUtil.exe does not require any extra command line
' parameters.



'test code below.. not used by application calling it.
CLS
DIM Config AS Configuration
Config.PopUpBgColor = 7
Config.PopUpFgColor = 0
Config.TitleBarFgColor = 15
Config.TitleBarBgColor = 10
Config.MenuBarBgColor = 10
Config.MenuBarFgColor = 15

DIM Body(1) AS STRING
Body(0) = "This is a test of the pop up"
Body(1) = "This is line 2 of the pop up. Woo"

CALL DrawPopUp(5, 5, "Pop Up Test", Body(), Config)

SUB ClearKbBuffer

DEF SEG = &H40
POKE &H1C, PEEK(&H1A)
DEF SEG

END SUB

FUNCTION DrawDirectoryList% (StartIndex AS INTEGER, SelectedIdx AS INTEGER, SubDirList() AS Directory, Config AS Configuration)

'display list

MaxDirShown% = 0
CurrentLineNum% = 4

'make sure idx is in range
IF StartIndex < 0 THEN
    StartIndex = 0
END IF

FOR i% = StartIndex TO UBOUND(SubDirList)

    IF CurrentLineNum% < Config.NumRows - 2 THEN

        'highlight selected
        IF SelectedIdx = SubDirList(i%).Id THEN
            COLOR Config.MainBgColor, Config.MainSelectedColor
        ELSE
            COLOR Config.MainFgColor, Config.MainBgColor
        END IF

        LOCATE CurrentLineNum%, 1
        PRINT "["; SubDirList(i%).DirName; "]"
       
        StrLen% = LEN(SubDirList(i%).DirName) + 3
        LOCATE CurrentLineNum%, StrLen%
        PRINT SPACE$(20 - StrLen%)

        CurrentLineNum% = CurrentLineNum% + 1
        MaxDirShown% = i%
      
    END IF

NEXT i%

'Mask out any unused space after list is shown
COLOR Config.MainBgColor, Config.MainBgColor
FOR i% = CurrentLineNum% TO Config.NumRows - 2
    LOCATE i%, 1
    PRINT SPACE$(19)
NEXT i%

IF DEBUG THEN

    LOCATE 4, 50
    PRINT "Start IDX: "; StartIndex

    LOCATE 5, 50
    PRINT "Max idx shown (retval): "; MaxDirShown%
END IF

'+1 causes some nifty scrolling... keep in mind.
DrawDirectoryList = MaxDirShown% + 1

END FUNCTION

SUB DrawDirScrollBar (SelectedIdx AS INTEGER, TotalDirs AS INTEGER, NumRows AS INTEGER)

'draw screen split between dirs and files

'top up arrow
COLOR 0, 7
LOCATE 4, 20
PRINT CHR$(24)


'bottom down arrow
LOCATE NumRows - 2, 20
PRINT CHR$(25)

LinesInScrollBar% = NumRows - 8

'16 lines in scroll bar... selected/subdircount = x/16
'selected * 16 = subdircount * x
'x = ((selected * 16) / subdircount) + yPosOffset

ySelected% = 5

'make sure there's sub dirs to divide against.
IF TotalDirs > 0 THEN
    ySelected% = ((SelectedIdx * LinesInScrollBar%) / TotalDirs) + 5
END IF
   

IF DEBUG THEN
    LOCATE 8, 24
    PRINT "SelectedIdx: "; SelectedIdx

    LOCATE 9, 24
    PRINT "TotalDirs: "; TotalDirs

    LOCATE 10, 24
    PRINT "ySelected: "; ySelected%
END IF

'scroll line
COLOR 7, 0
FOR y% = 5 TO NumRows - 3
    LOCATE y%, 20

    IF y% = ySelected% THEN
        PRINT CHR$(178)
    ELSE
        PRINT CHR$(176)
    END IF
NEXT y%



END SUB

SUB DrawFileList (SelectedIdx AS INTEGER, FileItemList() AS FileItem, Config AS Configuration)

'offset used to position file text on screen
'another magic number based on scroll bar x pos
DIM xOffset AS INTEGER
xOffset = 21

HeaderOffset% = 4


MaxLine% = Config.NumRows - 2
MaskString$ = SPACE$(59)


'get min display idx and make sure it'positive
MinDisplayIdx% = 0
IF SelectedIdx - 9 > 0 THEN
    MinDisplayIdx% = SelectedIdx - 9
END IF



'display file list section based on selected idx.
CurrentLine% = HeaderOffset%

FOR i% = CurrentLine% TO MaxLine%

    IF i% - HeaderOffset% + MinDisplayIdx% < UBOUND(FileItemList) THEN

        CurIdx% = i% - HeaderOffset% + MinDisplayIdx%

        IF CurIdx% = SelectedIdx THEN
            COLOR Config.MainBgColor, Config.MainSelectedColor

        ELSE
            COLOR Config.MainFgColor, Config.MainBgColor
           
        END IF

        LOCATE i%, xOffset
        PRINT FileItemList(CurIdx%).DisplayString

    ELSE
        LOCATE i%, xOffset
        COLOR Config.MainBgColor, Config.MainBgColor
        PRINT MaskString$

    END IF

NEXT i%



END SUB

DEFINT A-Z
SUB DrawFileScrollBar (SelectedIdx AS INTEGER, TotalFiles AS INTEGER, NumRows AS INTEGER)


'draw file scroll bar


x% = 80 ' TODO : Base this on screen res!

'top up arrow
COLOR 0, 7
LOCATE 4, x%
PRINT CHR$(24)


'bottom down arrow
LOCATE NumRows - 2, x%
PRINT CHR$(25)


LinesInScrollBar% = NumRows - 8


'16 lines in scroll bar... selected/filecount = x/16
'selected * 16 = filecount * x
'x = ((selected * 16) / filecount) + yPosOffset

ySelected% = 5

'make sure there's file items to divide against.
IF TotalFiles > 0 THEN
    ySelected% = ((SelectedIdx * LinesInScrollBar%) / (TotalFiles - 1)) + 5
END IF
  

IF DEBUG THEN
    LOCATE 8, 24
    PRINT "SelectedIdx: "; SelectedIdx

    LOCATE 9, 24
    PRINT "TotalFiles: "; TotalFiles

    LOCATE 10, 24
    PRINT "ySelected: "; ySelected%
END IF

'scroll line
COLOR 7, 0
FOR y% = 5 TO NumRows - 3
    LOCATE y%, x%

    IF y% = ySelected% THEN
        PRINT CHR$(178)
    ELSE
        PRINT CHR$(176)
    END IF
NEXT y%



END SUB

SUB DrawMenu (x AS INTEGER, Options() AS MenuItem, AreaSelected AS INTEGER, Config AS Configuration)

DIM h AS INTEGER
DIM w AS INTEGER
DIM y AS INTEGER
DIM yMax AS INTEGER


y = 3
h = UBOUND(Options) + 2
yMax = y + h
w = 0

'get max width of menu
FOR i% = 0 TO UBOUND(Options)
    IF LEN(RTRIM$(Options(i%).text)) > w THEN
        w = LEN(RTRIM$(Options(i%).text))
    END IF
NEXT i%

'width padding one char on each side of largest menu option
w = w + 2



FOR yCur% = y TO yMax + 1
    IF yCur% <= yMax THEN


        'background of menu with left and right borders.
        LOCATE yCur%, x
        COLOR 0, Config.PopUpBgColor
        PRINT CHR$(179) + SPACE$(w - 1) + CHR$(179)

        LineToDraw$ = ""
        FOR xCur% = x TO x + w
           
            LOCATE yCur%, xCur%

            'top bordr
            IF yCur% = y THEN
                COLOR 0, Config.PopUpBgColor
                IF xCur% = x THEN
                    'left top edge border
                    LineToDraw$ = LineToDraw$ + CHR$(218)

                ELSEIF xCur% = x + w THEN
                    'right top edge border
                    LineToDraw$ = LineToDraw$ + CHR$(191)
                ELSE
                    'top center border
                    LineToDraw$ = LineToDraw$ + CHR$(196)
                END IF


            'bottom border
            ELSEIF yCur% = yMax THEN
                COLOR 0, Config.PopUpBgColor
                IF xCur% = x THEN
                    'left bottom edge border
                    LineToDraw$ = LineToDraw$ + CHR$(192)

                ELSEIF xCur% = x + w THEN
                    'right bottom edge border
                    LineToDraw$ = LineToDraw$ + CHR$(217)
                ELSE
                    'bottom center border
                    LineToDraw$ = LineToDraw$ + CHR$(196)
                END IF


            END IF
        NEXT xCur%

        'draw current line borders
        LOCATE yCur%, x
        COLOR 0, Config.PopUpBgColor
        PRINT LineToDraw$


        'side shadow
        IF yCur% <> y THEN
            COLOR 8, 0
            LOCATE yCur%, xCur%
            PRINT CHR$(219)
           
        END IF


    'bottom shadow
    ELSE
      
        FOR xCur% = x + 1 TO x + w + 1
            LOCATE yCur%, xCur%
            COLOR 8, 0
            PRINT CHR$(223)
           
        NEXT xCur%
    END IF

NEXT yCur%



'print menu list with formatting for select character.
FOR opt% = 0 TO UBOUND(Options)
 
    IF IsMenuItemEnabled(Options(opt%).Id, AreaSelected, Config.DosBoxCompMode) THEN

        'menu divider ids always drawn the same.
        IF Options(opt%).Id = MENUIDDIVIDER THEN

            text$ = CHR$(195)
            FOR i% = 0 TO w - 2
                text$ = text$ + CHR$(196)
            NEXT i%
            text$ = text$ + CHR$(180)

            COLOR Config.PopUpFgColor, Config.PopUpBgColor
            LOCATE y + opt% + 1, x
            PRINT text$

        ELSE
            'all other enabled menu items fall into this.

            text$ = RTRIM$(Options(opt%).text)

            'sort of a hack for toggling text for hidden
            'files menu option...
            IF Options(opt%).Id = MENUIDOPTIONHIDDENFILES THEN
                IF NOT Config.ShowHiddenFiles THEN
                    text$ = SHOWHIDDENFILESTEXT
                ELSE
                    text$ = HIDEHIDDENFILESTEXT
                END IF

            END IF

            COLOR Config.PopUpFgColor, Config.PopUpBgColor
            LOCATE y + opt% + 1, x + 1
            PRINT text$

            'get highlighted letter and print it.
            COLOR 9, Config.PopUpBgColor
            LOCATE y + opt% + 1, x + Options(opt%).HighlightXOffset + 1
            PRINT Options(opt%).HighlightCharacter
        END IF

    ELSE
        'disabled menu option
        COLOR Config.MenuDisabledFgColor, Config.PopUpBgColor
        LOCATE y + opt% + 1, x + 1
        PRINT RTRIM$(Options(opt%).text)
    END IF

NEXT opt%


END SUB

DEFSNG A-Z
SUB DrawPopUp (x AS INTEGER, y AS INTEGER, Title AS STRING, Body() AS STRING, Config AS Configuration)

DIM h AS INTEGER
DIM w AS INTEGER

h = UBOUND(Body) + 1
w = 0

'get max width of pop up base on longest body line
FOR i% = 0 TO UBOUND(Body)
    IF LEN(Body(i%)) > w THEN
        w = LEN(Body(i%))
    END IF
NEXT i%

MaxWidth% = x + w + 2
IF MaxWidth% > Config.NumColumns THEN
    MaxWidth% = Config.NumColumns
END IF

'if title is longer, use that for width.
IF LEN(Title) > w THEN
    w = LEN(Title)
END IF

'width padding one char on each side of largest body string
w = w + 2


'draw pop up window
COLOR Config.TitleBarBgColor, Config.TitleBarBgColor
LOCATE y, x
PRINT SPACE$(MaxWidth% - x)


'draw window body with shadow

FOR yCur% = y + 1 TO y + h + 1

    IF yCur% <= y + h THEN

        'main background
        COLOR Config.PopUpBgColor, Config.PopUpBgColor
        LOCATE yCur%, x
        PRINT SPACE$(MaxWidth% - x)

        LOCATE yCur%, MaxWidth%
        COLOR 8, 0
        PRINT CHR$(219)

    'bottom shadow
    ELSE
        FOR xCur% = x + 1 TO MaxWidth%
            LOCATE yCur%, xCur%
            COLOR 8, 0
            PRINT CHR$(223)
        NEXT xCur%
    END IF

NEXT yCur%


'add title bar text
COLOR Config.TitleBarFgColor, Config.TitleBarBgColor
LOCATE y, x + 1
PRINT Title

'add body text
COLOR Config.PopUpFgColor, Config.PopUpBgColor
FOR BodyText% = 0 TO UBOUND(Body)
    LOCATE y + BodyText% + 1, x + 1
    PRINT Body(BodyText%)
NEXT BodyText%


END SUB

SUB DrawScr (CurPath AS STRING, Config AS Configuration)

'top banner
COLOR Config.TitleBarBgColor, Config.TitleBarBgColor
LOCATE 1, 1
PRINT SPACE$(80)

'title text
COLOR Config.TitleBarFgColor, Config.TitleBarBgColor
IF Config.XtCompMode THEN
    LOCATE 1, 18
    PRINT "NE File Utility "; VER; " (PC/XT Compatibility Mode)"
ELSE
    LOCATE 1, 30
    PRINT "NE File Utility "; VER
END IF

'file menu bar
COLOR Config.MenuBarBgColor, Config.MenuBarBgColor
LOCATE 2, 1
PRINT SPACE$(80)

'Current Directory bar
COLOR Config.MenuBarBgColor, Config.MenuBarBgColor
LOCATE 3, 1
PRINT SPACE$(80)

'draw menu options as not selected.
CALL ToggleMenuFocus(FALSE, CurPath, Config)

END SUB

DEFINT A-Z
FUNCTION FileExists% (FilePath AS STRING)

FileExists = FALSE

f% = FREEFILE

'ON ERROR GOTO ErrorHandler

RemoveTempFile% = FALSE

OPEN FilePath FOR APPEND AS #f%

    IF LOF(f%) THEN
        FileExists = TRUE
    ELSE
        FileExists = FALSE
        RemoveTempFile% = TRUE
    END IF

CLOSE #f%


'sort of daring and may be removed as it assumes 0 byte files
'should be deleted when that may not be the case.
IF RemoveTempFile% THEN
    KILL FilePath
END IF

END FUNCTION

FUNCTION GetDosVersion$

'Gets current version of dos running.

SHELL "VER > " + TEMPVERFILENAME

DosVer$ = "5.00"

OPEN TEMPVERFILENAME FOR INPUT AS #1
    WHILE NOT EOF(1)
        INPUT #1, line$
        IF LTRIM$(RTRIM$(line$)) <> "" THEN
            DosVer$ = line$
        END IF
    WEND
CLOSE #1

GetDosVersion$ = LTRIM$(RTRIM$(DosVer$))

END FUNCTION

FUNCTION IsMenuItemEnabled% (MenuItemId AS INTEGER, AreaSelected AS INTEGER, IsDosBoxMode AS INTEGER)

'Checks if a menu option should be enabled or not based
'on the current application configuration.

isEnabled% = TRUE

SELECT CASE MenuItemId

    'Currently don't allow file functions on DIR. will change
    'in the future.
    CASE MENUIDFILERENAME, MENUIDFILECOPY, MENUIDFILEMOVE
        IF AreaSelected = AREADIR THEN
            isEnabled% = FALSE
        END IF

    'extended dir options missing in DosBox
    CASE MENUIDOPTIONHIDDENFILES, MENUIDOPTIONSORTORDER

        IF IsDosBoxMode THEN
            isEnabled% = FALSE
        END IF

END SELECT

IsMenuItemEnabled% = isEnabled%

END FUNCTION

SUB ToggleMenuFocus (IsOn AS INTEGER, CurPath AS STRING, Config AS Configuration)

'Draw main text by default

'File
COLOR Config.MenuBarFgColor, Config.MenuBarBgColor
LOCATE 2, 4
PRINT "File"

'Options
LOCATE 2, 12
PRINT "Options"

'Help
LOCATE 2, 70
PRINT "Help"

LOCATE 3, 4
PRINT "Current Path: "; CurPath

'Color navigation character if focus set to menu

IF IsOn THEN
    COLOR 9, Config.MenuBarBgColor
    LOCATE 2, 4
    PRINT "F"

    LOCATE 2, 12
    PRINT "O"

    LOCATE 2, 70
    PRINT "H"

    LOCATE 3, 4
    PRINT "C"
END IF

END SUB

